﻿' ******************************************
' By Bernardo Giovanni - 2009
' http://www.settorezero.com
' Esempio di utilizzo Firebird da vb.net
' ******************************************

' ATTENZIONE !!
' Il seguente codice è protetto dalla legge sul
' diritto d'autore.
' Non è consentito ridistribuire questo codice 
' sorgente di esempio in nessun modo e deve essere
' utilizzato unicamente a scopo didattico o per
' uso personale. Altri utilizzi non consentiti, come
' la modifica a scopo di lucro, saranno denunciati.
' Sono gradite segnalazioni di malfunzionamento
' e/o modifiche e migliorie apportate al codice stesso

Imports FirebirdSql.Data.Firebird ' Importo lo spazio dei nomi di Firebird

Public Class frmMain

    Private Sub frmMain_Load(ByVal sender As System.Object, _
                             ByVal e As System.EventArgs) _
                             Handles MyBase.Load

        ' All'avvio riempio la DataGridView con i dati provenienti
        ' dal database, sfruttando l'apposita sub "RefreshData"
        RefreshData()

    End Sub

    Private Sub btnExit_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) _
                              Handles btnExit.Click

        ' Click sul pulsante "Uscita"
        Me.Close()

    End Sub

    Private Sub btnNew_Click(ByVal sender As System.Object, _
                             ByVal e As System.EventArgs) _
                             Handles btnNew.Click

        ' Click sul pulsante "Nuovo"

        ' Setto la mia variabile globale "IdDaModificare" sul valore -1
        ' così da far capire a frmEdit che devo eseguire l'inserimento
        ' di un nuovo record e non la modifica di un record esistente
        IdDaModificare = -1
        frmEdit.ShowDialog()

        ' Alla chiusura di frmEdit, aggiorno i dati nella DataGridView
        RefreshData()

    End Sub

    Private Sub Edit(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) _
                              Handles btnEdit.Click, dgvRubrica.DoubleClick

        ' Click sul pulsante "Modifica" oppure doppio Click
        ' sulla DataGridView (da notare che questa sub difatti
        ' gestisce (handles) due eventi: il Click su btnEdit
        ' e il doppio click sulla DataGridView)

        ' Setto la mia variabile globale "IdDaModificare" sul valore
        ' corrispondente all'id del record da modificare.

        ' Attenzione: nel caso non ci siano righe nella DataGridView, il codice
        ' ovviamente solleva un'eccezione. Anche se è vero che se non
        ' ci sono righe non è possibile premere il tasto "Modifica" (viene difatti
        ' disattivato nella sub RefreshData), è comunque possibile fare doppio click
        ' sulla DataGridView e quindi questa sub viene eseguita, per cui dobbiamo
        ' controllare che la griglia contenga almeno una riga.

        If dgvRubrica.RowCount > 0 Then

            ' L'ID del record si trova nella colonna 0 (Cells(0)) della riga selezionata

            ' La DataGridView ha una proprietà che si chiama SelectedRows che restituisce
            ' l'insieme delle righe selezionate. Per cui per recuperare una riga tra le
            ' tante selezionate, è necessario fornire l'indice a tale proprietà, ovviamente
            ' fornendo indice 0 viene restituita la prima riga dell'insieme.

            ' Nel nostro caso, la DataGridView ha la proprietà MultiSelect impostata su
            ' FALSE, per cui non è possibile selezionare più di una riga, devo dare comunque
            ' l'indice 0 alla proprietà SelectedRows per indicargli che deve restituirmi
            ' la prima riga dell'insieme (anche se costituito da una sola riga).
            IdDaModificare = dgvRubrica.SelectedRows(0).Cells(0).Value
            frmEdit.ShowDialog()

            ' Alla chiusura di frmEdit, aggiorno i dati nella DataGridView
            RefreshData()

        End If

    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, _
                                ByVal e As System.EventArgs) _
                                Handles btnDelete.Click

        ' Click sul pulsante "Cancella"

        ' Recupero nome e cognome per mostrarli nella messagebox
        Dim Nome As String = dgvRubrica.SelectedRows(0).Cells(2).Value & _
                             " " & dgvRubrica.SelectedRows(0).Cells(1).Value


        ' Mostro la messagebox per richiedere la conferma della cancellazione
        If MessageBox.Show("Sei sicuro di voler cancellare l'utente " & Nome & " ?", _
                           "Cancellazione di un record", MessageBoxButtons.YesNo, _
                           MessageBoxIcon.Asterisk) = Windows.Forms.DialogResult.Yes Then

            ' è stato premuto il tasto "Si", effettuo la cancellazione

            ' Recupero l'id del campo attualmente selezionato
            IdDaModificare = dgvRubrica.SelectedRows(0).Cells(0).Value

            ' Strutturo la Query di cancellazione
            Dim Query As String = "DELETE FROM indirizzi WHERE id=" & IdDaModificare

            ' Mi collego al database
            Dim Connessione As New FbConnection(Str_Co)
            Connessione.Open()

            ' Definisco una variabile che mi servirà a memorizzare
            ' il risultato della Query
            Dim Risultato As Integer = 0

            ' Racchiudo l'esecuzione della Query in un blocco Try - Catch
            ' per intercettare eventuali eccezioni
            Try

                ' Definisco un comando
                ' Tale comando deve eseguire la mia Query
                Dim Comando As New FbCommand(Query, Connessione)

                ' Eseguo il comando e ne memorizzo il valore di uscita
                Risultato = Comando.ExecuteNonQuery

            Catch ex As Exception ' Catturo un'eventuale eccezione

                MessageBox.Show("Query fallita: " & vbCrLf & Query & vbCrLf & "Eccezione:" & _
                                vbCrLf & ex.ToString, "Errore", MessageBoxButtons.OK, _
                                MessageBoxIcon.Error)

            End Try

            ' Chiudo la connessione
            Connessione.Close()

            ' Se l'esecuzione del comando ha restituito zero come
            ' risultato, mostro un messaggio
            If Risultato = 0 Then

                MessageBox.Show("La Query non è andata a buon fine", "Errore", _
                                MessageBoxButtons.OK, MessageBoxIcon.Error)

            Else
                ' La query è andata a buon fine, aggiorno la visualizzazione
                ' della tabella
                RefreshData()

            End If

        End If

    End Sub

    Private Sub lnk_LinkClicked(ByVal sender As System.Object, _
                                ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) _
                                Handles lnk.LinkClicked

        ' Click sulla label dell'URL
        System.Diagnostics.Process.Start("http://www.settorezero.com?Rubrica_Firebird")

    End Sub

    Private Sub RefreshData()

        ' Questa sub aggiorna i dati visualizzati nella DataGridView
        ' viene richiamata all'avvio e quando viene chiuso il form
        ' di editing

        ' Disabilito momentaneamente il form e visualizzo
        ' il cursore con la clessidra
        Me.Enabled = False
        Me.Cursor = Cursors.WaitCursor

        ' Svuoto la DataGridView, eliminando le righe una ad una
        While dgvRubrica.RowCount > 0
            dgvRubrica.Rows.RemoveAt(0)
        End While
        dgvRubrica.Refresh()

        ' Definisco la query, effettuo l'ordinamento per cognome crescente
        Dim Query As String = "SELECT * FROM indirizzi ORDER BY cognome ASC"

        ' Creo la connessione al database
        Dim Co As New FbConnection

        ' Imposto la stringa di connessione, che ho definito nel modulo variabili
        Co.ConnectionString = Str_Co

        ' Creo l'adattatore e lo riempio coi dati della query
        Dim Adattatore As New FbDataAdapter(Query, Co)

        ' Creo una Tabella dati che conterrà i dati provenienti dall'adattatore
        Dim Tabella As New DataTable

        ' Dico all'adattatore di riempire la tabella con i suoi dati
        Adattatore.Fill(Tabella)

        ' Imposto la sorgente dati della DataGridView
        dgvRubrica.DataSource = Tabella

        ' Di default, l'intestazione delle colonne della DataGridView
        ' sarà il nome stesso dei campi, se vogliamo modificarlo, possiamo
        ' settare la proprietà HeaderText delle colonne:
        dgvRubrica.Columns(0).HeaderText = "id"
        dgvRubrica.Columns(1).HeaderText = "Nome"
        dgvRubrica.Columns(2).HeaderText = "Cognome"
        dgvRubrica.Columns(3).HeaderText = "Indirizzo E-Mail"

        ' Dato che non mi piace visualizzare la colonna ID, la nascondo
        ' (la colonna è comunque sempre presente ed è possibile prelevarne i dati)
        dgvRubrica.Columns(0).Visible = False

        ' Rilascio le risorse
        Adattatore.Dispose()
        Tabella.Dispose()

        ' Se non ci sono righe nella DataGridView, disabilito i tasti "Modifica" e "Cancella"
        If dgvRubrica.RowCount = 0 Then

            btnEdit.Enabled = False
            btnDelete.Enabled = False

        Else ' Altrimenti li attivo

            btnEdit.Enabled = True
            btnDelete.Enabled = True

        End If

        ' Riabilito il form e reimposto il cursore
        Me.Enabled = True
        Me.Cursor = Cursors.Arrow

    End Sub

End Class
